# encoding: utf-8
"""
@project: djangoModel->subitem_extend_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 分项扩展字段处理服务
@created_time: 2022/10/18 16:55
"""

# 分项扩展字段
import re

from ..models import Enroll, EnrollSubitemExtendField
from ..utils.custom_tool import format_params_handle


def output_convert(result_list=None):
    """结果集 扩展字段处理函数"""
    if not result_list:
        return []
    extend_field_list = list(EnrollSubitemExtendField.objects.all().values("category_id", "field_index", "field"))
    extend_field_map = {}
    for item in extend_field_list:
        extend_field_map.setdefault(item["category_id"], {})
        extend_field_map[item["category_id"]].update({item["field_index"]: item["field"]})

    result = []
    for item in result_list:
        this_category_map = extend_field_map.get(item.get("category_id") or 0, {})
        temp = {this_category_map.get(k, k): v for k, v in item.items()}
        result.append({k: v for k, v in temp.items() if not re.search("field_.*", k)})
    return result


def input_convert(params_dict=None, enroll_id=None):
    """修改或插入数据时 参数还原成数据库对应字段"""
    if not params_dict or not enroll_id:
        return {}

    enroll_obj = Enroll.objects.filter(id=enroll_id)
    category_id = enroll_obj.first().to_json().get("category_id") if enroll_obj else None
    if not category_id:
        return format_params_handle(
            param_dict=params_dict,
            filter_filed_list=["enroll_id", "name", "price", "count", "unit", "amount", "description", "remark", "enroll_subitem_status_code"]
        )

    extend_field_obj = EnrollSubitemExtendField.objects.filter(category_id=category_id)
    extend_field_list = extend_field_obj.values("field_index", "field") if extend_field_obj else []
    extend_field_map = {item["field"]: item["field_index"] for item in extend_field_list}
    return format_params_handle(
        param_dict=params_dict,
        filter_filed_list=["enroll_id", "name", "price", "count", "unit", "amount", "description", "remark", "enroll_subitem_status_code"] + list(extend_field_map.keys()),
        alias_dict=extend_field_map
    )



from django.db import transaction
from django.views.decorators.http import require_http_methods
from rest_framework.decorators import api_view
from rest_framework.views import APIView

from xj_thread.services.thread_category_service import ThreadCategoryService
from xj_thread.services.thread_item_service import ThreadItemService
from xj_thread.services.thread_list_service import ThreadListService
from xj_user.services.user_detail_info_service import DetailInfoService
from xj_user.utils.user_wrapper import user_authentication_force_wrapper, user_authentication_wrapper
from ..service.clock_service import ClockService
from ..service.enroll_record_serivce import EnrollRecordServices
from ..service.enroll_services import EnrollServices
from ..service.enroll_subitem_record_service import EnrollSubitemRecordService
from ..service.rule_service import RuleValueService
from ..service.subitem_service import SubitemService
from ..utils.custom_response import util_response
from ..utils.custom_tool import parse_data, format_params_handle, request_params_wrapper, filter_result_field, flow_service_wrapper, write_to_log
from ..utils.join_list import JoinList


class EnrollAPI(APIView):
    @staticmethod
    def list_handle(*args, request_params=None, user_info=None, **kwargs, ):
        """
        接口执行手柄
        :param request_params: request请求的参数
        :param user_info: 请求的用户信息
        :return: data,err
        """
        # 信息id列表反查询报名
        thread_ids, err = ThreadListService.search_ids(search_prams=format_params_handle(
            param_dict=request_params,
            filter_filed_list=["title", "subtitle", "access_level", "author"],
            is_remove_empty=True
        ))
        if thread_ids:
            request_params["thread_id_list"] = thread_ids

        # 根据平台编码获取类别ID列表
        platform_code = request_params.pop("platform_code", None)
        if platform_code:
            id_list, err = ThreadCategoryService.list(
                filter_fields_params={"platform_code": platform_code},
                filter_fields="id",
                need_pagination=False
            )
            request_params["category_id_list"] = [i["id"] for i in id_list]

        # 报名列表搜索
        data, err = EnrollServices.enroll_list(params=request_params)
        if err:
            return None, err

        # 合并信息表数据
        id_list = [i['thread_id'] for i in data['list'] if i]
        thread_list, err = ThreadListService.search(id_list)
        data['list'] = JoinList(data['list'], thread_list, "thread_id", "id").join()
        # 用户信息合并
        user_id_list = [i['user_id'] for i in data['list'] if i]
        user_info_list = DetailInfoService.get_list_detail(user_id_list=user_id_list, filter_fields=["user_id", "real_name", "full_name", "nickname"])
        data['list'] = JoinList(data['list'], user_info_list, "user_id", "user_id").join()

        # 小数点四舍五入
        for i in data['list']:
            i["price"] = round(i["price"], 2)

        # 结果集字段过滤 TODO 此处应该给默认的 字段列表
        filter_fields = request_params.get("filter_fields", None)
        filter_fields = filter_fields.split(";") if filter_fields else None
        data["list"] = filter_result_field(result_list=data["list"], filter_filed_list=filter_fields)
        # 数据返回
        if err:
            return None, err
        return data, None

    @api_view(['GET'])
    @request_params_wrapper
    @user_authentication_wrapper
    def list(self, *args, request_params=None, user_info=None, **kwargs, ):
        data, err = EnrollAPI.list_handle(request_params=request_params, user_info=user_info, )
        # 数据返回
        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data)

    @api_view(['POST', ])
    @user_authentication_force_wrapper
    @request_params_wrapper
    @flow_service_wrapper
    def add(self, *args, user_info, request_params, **kwargs, ):
        params = request_params
        # ============   字段验证处理 start ============
        user_id = user_info.get("user_id")
        params.setdefault("user_id", user_id)  # 用户ID

        params.setdefault("subitems", [])  # 报名分项
        enroll_subitem_status_code = params.pop("enroll_subitem_status_code", 242)  # 报名分项 状态码

        # ============ 如果有计价id，并且需要重新计价，补全补全的参数  start ============
        # try:
        #     is_valuate = int(params.pop("is_valuate", 1))  # 是否重新验单，进行计价，补全没有传的参数
        # except Exception:
        #     is_valuate = 0

        # if is_valuate and main_params.get("enroll_rule_group_id"):
        #     print("params:", params)
        # ============ 如果有计价id，并且需要重新计价，补全补全的参数  end ============

        # 信息表参数
        thread_params = format_params_handle(
            param_dict=params,
            remove_filed_list=[
                # 报名表字段
                "trading_relate", "region_code", "occupy_room", "enroll_status_code", "min_number", "max_number", "min_count_apiece", "max_count_apiece",
                "enroll_rule_group", "price", "count", "unit", "fee", "reduction", "subitems_amount", "amount", "paid_amount", "unpaid_amount", "commision",
                "deposit", "hide_price", "hide_user", "has_repeat", "has_subitem", "has_audit", "need_vouch", "need_deposit", "need_imprest", "enable_pool",
                "pool_limit", "pool_stopwatch", "open_time", "close_time", "launch_time", "finish_time", "spend_time", "create_time",
                "update_time", "snapshot", "remark", "finance_invoicing_code",
                # 报名分项字段
                "subitem"
            ],
            alias_dict={"thread_remark": "remark"}
        )
        if not thread_params:
            return util_response(err=1001, msg="请填写项目基本信息")
        thread_params["has_enroll"] = 1  # 默认参数开启报名
        # 报名主表参数
        main_params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "thread_id", "user_id", "category_id", "trading_relate", "region_code", "occupy_room", "enroll_status_code", "min_number", "max_number",
                "min_count_apiece", "max_count_apiece", "enroll_rule_group", "enroll_rule_group_id", "price", "count", "unit", "fee", "reduction", "subitems_amount",
                "amount", "paid_amount", "unpaid_amount", "commision", "deposit", "hide_price", "hide_user", "has_repeat", "has_subitem", "has_audit",
                "need_vouch", "need_deposit", "need_imprest", "enable_pool", "pool_limit", "pool_stopwatch", "open_time", "close_time", "launch_time",
                "finish_time", "spend_time", "create_time", "update_time", "snapshot", "remark", "finance_invoicing_code",
            ],
        )
        if thread_params:
            main_params.setdefault("snapshot", thread_params)  # 因为信息表需要调用服务，所以存一组信息表的快照
        # ============   字段验证处理 end ============

        # ============   绑定计价分组ID start ============
        try:
            if not main_params.get("enroll_rule_group_id"):
                # 得到 enroll_rule_group_id
                category_id = thread_params.get("category_id", None)
                classify_id = thread_params.get("classify_id", None)
                if not category_id or not classify_id:
                    raise Exception("category_id，classify_id必填")
                res, err = RuleValueService.group_list({"category_id": category_id, "classify_id": classify_id})
                if err:
                    raise Exception(err)
                if not res:
                    raise Exception("没有找到对应的计价规则组ID")
                main_params.setdefault("enroll_rule_group_id", res[0].get("id"))
        except Exception as e:
            write_to_log(prefix="报名发布，绑定计价组ID异常", err_obj=e)
        # ============   绑定计价分组ID end ============

        # ============ 信息表添加  start ============
        sid = transaction.savepoint()
        thread_id = thread_params.get("thread_id")
        if not thread_id:
            data, err = ThreadItemService.add(thread_params)
        else:
            data, err = ThreadItemService.edit(thread_params, thread_id)
        if err:
            transaction.savepoint_rollback(sid)
            return util_response(err=1002, msg="信息添加错误：" + err)
        # ============ 信息表添加  end   ============

        # ============ 报名主表添加  start ============
        thread_id = thread_id or data['id']
        main_params.setdefault("thread_id", thread_id or data['id'])
        main_params.setdefault("enroll_status_code", 242)
        main_instance, err = EnrollServices.enroll_add(main_params)
        if err:
            transaction.savepoint_rollback(sid)
            return util_response(err=1003, msg=err)
        # ============ 报名主表添加  end ============

        # ============ 报名分项添加  start ============
        for item_params in params.get("subitems", []):
            item_params.setdefault("enroll_id", main_instance.get('id'))
            item_params.setdefault("enroll_subitem_status_code", enroll_subitem_status_code)
            data, err = SubitemService.add(item_params)
            if err:
                transaction.savepoint_rollback(sid)
                return util_response(err=1004, msg=err)
        # ============ 报名分项添加  end ============

        transaction.clean_savepoints()  # 清除保存点
        return util_response(data=main_instance)

    @api_view(['GET', 'POST', ])
    @user_authentication_force_wrapper
    @request_params_wrapper
    def detail(self, *args, user_info=None, request_params=None, **kwargs, ):
        # 参数验证
        if request_params is None:
            request_params = {}
        if user_info is None:
            user_info = {}

        params = request_params
        default_remove_fields = [
            "real_name", "sex", "birth", "tags", "signature", "avatar", "cover", "language", "more",
            "id_card_front", "id_card_back", "real_name_is_pass", "id_card",
            "certificate", "work experience", "qualification_certificate", "audit_status",
            "score", "user_name", "phone", "email", "register_time",
            "user_info", "wechat_openid", "is_deleted",
        ]
        filter_fields = params.pop("filter_fields", None)
        remove_fields = params.pop("remove_fields", default_remove_fields)

        enroll_id = kwargs.get("enroll_id", None) or params.pop("enroll_id", None)
        if not enroll_id:
            return util_response(err=1000, msg="参数错误:enroll_id不可以为空")

        user_id = user_info.get("user_id", None)

        # 获取用户详情
        data, err = EnrollServices.enroll_detail(enroll_id, user_id=user_id)
        if err:
            return util_response(err=1001, msg=err)

        # 用户数据拼接
        push_user_info, user_err = DetailInfoService.get_detail(user_id=user_id)
        if push_user_info:
            data.update(push_user_info)

        # 信息表 数据拼接
        thread_list, err = ThreadListService.search([data.get("thread_id")])
        if err:
            return util_response(err=1002, msg=err)
        data = JoinList([data], thread_list, "thread_id", "id").join()
        data = data[0] if not len(data) > 1 else data
        data["price"] = round(data["price"] or 0, 2)

        # 字段过滤
        filter_fields = filter_fields.split(";") if filter_fields and isinstance(filter_fields, str) else filter_fields
        remove_fields = remove_fields.split(";") if remove_fields and isinstance(remove_fields, str) else remove_fields
        data = format_params_handle(data, filter_filed_list=filter_fields, remove_filed_list=remove_fields, is_remove_null=False)
        # 响应数据
        if err:
            return util_response(err=1003, msg=err)
        return util_response(data=data)

    @require_http_methods(['GET'])
    @request_params_wrapper
    @user_authentication_force_wrapper
    def own_list(self, *args, request_params=None, user_info=None, **kwargs):
        # 信息表
        thread_params = format_params_handle(
            param_dict=request_params,
            filter_filed_list=[
                "user_id", "title", "subtitle", "summary", "access_level", "author", "has_enroll", "has_fee", "has_comment", "has_location", "is_original",
            ]
        )
        if thread_params:
            thread_ids, err = ThreadListService.search_ids(thread_params)
            request_params["thread_id_list"] = thread_ids if not err else []

        filter_fields = request_params.pop("filter_fields", None)
        request_params["user_id"] = user_info.get("user_id")
        need_pagination = request_params.pop("need_pagination", 1)
        need_pagination = int(need_pagination)
        data, err = EnrollServices.enroll_own_list(request_params, need_pagination=need_pagination)
        if err:
            return util_response(err=1000, msg=err)
        # 合并thread模块数据
        id_list = [i['thread_id'] for i in data['list'] if i]
        thread_list, err = ThreadListService.search(id_list)
        data['list'] = JoinList(data['list'], thread_list, "thread_id", "id").join()

        filter_fields = filter_fields.split(";") if filter_fields else None
        data["list"] = filter_result_field(result_list=data["list"], filter_filed_list=filter_fields)
        # 数据返回
        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data)

    @api_view(['GET', 'POST', ])
    @request_params_wrapper
    @user_authentication_force_wrapper
    def undertake_list(self, *args, request_params=None, user_info=None, **kwargs):
        """个人承办列表, 表示的报名记录列表"""
        filter_fields = request_params.get("filter_fields", None)
        request_params["user_id"] = user_info.get("user_id")
        need_pagination = request_params.pop("need_pagination", 1)
        need_pagination = int(need_pagination)
        # 信息表
        thread_params = format_params_handle(
            param_dict=request_params,
            filter_filed_list=[
                "title", "subtitle", "access_level", "author", "has_enroll", "has_fee", "has_comment",
                "has_location", "is_original",
            ]
        )
        if thread_params:
            thread_ids, err = ThreadListService.search_ids(thread_params)
            request_params["thread_id_list"] = thread_ids if not err else []
        data, err = EnrollServices.enroll_undertake_list(request_params, need_pagination=need_pagination)
        if err:
            return util_response(err=1000, msg=err)
        # 分页数据
        if need_pagination:
            for i in data["list"]:
                i['price'] = round(float(i["price"]), 2)
                i['main_amount'] = round(float(i["main_amount"]), 2)
                i['coupon_amount'] = round(float(i["coupon_amount"]), 2)
                i['again_reduction'] = round(float(i["again_reduction"]), 2)
                i['subitems_amount'] = round(float(i["subitems_amount"]), 2)
                i['deposit_amount'] = round(float(i["deposit_amount"]), 2)
                i['amount'] = round(float(i["amount"]), 2)
                i['paid_amount'] = round(float(i["paid_amount"]), 2)
                i['unpaid_amount'] = round(float(i["unpaid_amount"]), 2)
                i['fee'] = round(float(i["fee"]), 2)

            # 合并thread模块数据
            id_list = [i['thread_id'] for i in data['list'] if i]
            thread_list, err = ThreadListService.search(id_list)
            data['list'] = JoinList(data['list'], thread_list, "thread_id", "id").join()

            filter_fields = filter_fields.split(";") if filter_fields else None
            data["list"] = filter_result_field(result_list=data["list"], filter_filed_list=filter_fields)
            # 数据返回
            if err:
                return util_response(err=1001, msg=err)
            return util_response(data=data)
        else:
            for i in data:
                i['price'] = round(float(i["price"]), 2)
                i['main_amount'] = round(float(i["main_amount"]), 2)
                i['coupon_amount'] = round(float(i["coupon_amount"]), 2)
                i['again_reduction'] = round(float(i["again_reduction"]), 2)
                i['subitems_amount'] = round(float(i["subitems_amount"]), 2)
                i['deposit_amount'] = round(float(i["deposit_amount"]), 2)
                i['amount'] = round(float(i["amount"]), 2)
                i['paid_amount'] = round(float(i["paid_amount"]), 2)
                i['unpaid_amount'] = round(float(i["unpaid_amount"]), 2)
                i['fee'] = round(float(i["fee"]), 2)

            # 合并thread模块数据
            id_list = [i['thread_id'] for i in data if i]
            thread_list, err = ThreadListService.search(id_list)
            data = JoinList(data, thread_list, "thread_id", "id").join()

            filter_fields = filter_fields.split(";") if filter_fields else None
            data = filter_result_field(result_list=data, filter_filed_list=filter_fields)
            # 数据返回
            if err:
                return util_response(err=1001, msg=err)
            return util_response(data=data)

    @api_view(['POST', 'PUT'])
    @request_params_wrapper
    @flow_service_wrapper
    def edit(self, *args, request_params=None, **kwargs, ):
        # 参数处理
        if request_params is None:
            request_params = {}
        params = request_params.copy()
        enroll_id = kwargs.get("enroll_id") or params.pop("enroll_id") or None
        if not enroll_id:
            return util_response(err=1000, msg="参数错误:enroll_id不可以为空")
        thread_params = format_params_handle(
            param_dict=params,
            remove_filed_list=[
                "trading_relate", "user_id", "region_code", "occupy_room", "enroll_status_code", "min_number", "max_number",
                "min_count_apiece", "max_count_apiece", "enroll_rule_group", "price", "count", "unit", "fee", "reduction", "subitems_amount", "amount",
                "paid_amount", "unpaid_amount", "commision", "deposit", "hide_price", "hide_user", "has_repeat", "has_subitem", "has_audit", "need_vouch",
                "need_deposit", "need_imprest", "enable_pool", "pool_limit", "pool_stopwatch", "open_time", "close_time", "launch_time", "finish_time",
                "spend_time", "create_time", "update_time", "snapshot", "remark", "finance_invoicing_code"
            ],
            alias_dict={"thread_remark": "remark"}
        )

        main_params = format_params_handle(
            param_dict=params,
            filter_filed_list=[
                "thread_id", "category_id", "trading_relate", "region_code", "occupy_room", "enroll_status_code", "min_number", "max_number",
                "min_count_apiece", "max_count_apiece", "enroll_rule_group", "price", "count", "unit", "fee", "reduction", "subitems_amount", "amount",
                "paid_amount", "unpaid_amount", "commision", "deposit", "hide_price", "hide_user", "has_repeat", "has_subitem", "has_audit", "need_vouch",
                "need_deposit", "need_imprest", "enable_pool", "pool_limit", "pool_stopwatch", "open_time", "close_time", "launch_time", "finish_time",
                "spend_time", "create_time", "update_time", "snapshot", "remark",
            ]
        )
        if thread_params:
            main_params.setdefault("snapshot", thread_params)  # 因为信息表需要调用服务，所以存一组信息表的快照

        # 报名主表修改
        # print("main_params:", main_params)
        instance, err = EnrollServices.enroll_edit(main_params, enroll_id)
        if err:
            return util_response(err=1001, msg=err)

        # 信息表修改
        thread_id = params.pop("thread_id", None) or instance.get("thread_id", None)
        if thread_id and thread_params:
            data, err = ThreadItemService.edit(thread_params, thread_id)
            if err:
                return util_response(err=1002, msg=err)

        # 联动分项数量修改
        subitem__count = params.pop("subitem__count", None)
        if subitem__count:
            data, err = SubitemService.batch_edit({"count": subitem__count}, enroll_id=enroll_id)
            if err:
                return util_response(err=1003, msg=err)

        return util_response(data=instance)

    @require_http_methods(['DELETE'])
    def delete(self, *args, **kwargs, ):
        params = parse_data(self)
        enroll_id = kwargs.get("enroll_id") or params.pop("enroll_id") or None
        if not enroll_id:
            return util_response(err=1000, msg="参数错误:enroll_id不可以为空")
        data, err = EnrollServices.enroll_delete(enroll_id)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @api_view(['POST', 'PUT'])
    @request_params_wrapper
    @user_authentication_wrapper
    def enroll(self, *args, request_params=None, user_info=None, **kwargs):
        sid = transaction.savepoint()
        # 用户id获取
        request_params.setdefault("user_id", user_info.get("user_id", None))
        if not request_params.get("user_id"):
            return util_response(err=1000, msg="user_id 必填")
        # is_can_add = EnrollRecordServices.check_can_add(enroll_id=request_params.get("enroll_id"), user_id=user_info.get("user_id"))
        # if not is_can_add:
        #     return util_response(err=1001, msg="无法继续报名，当前人数已满")

        # 信息主表
        main_record_params = format_params_handle(
            param_dict=request_params,
            filter_filed_list=[
                "id", "enroll_id", "user_id", "enroll_auth_status_id", "enroll_pay_status_id", "enroll_status_code", "create_time", "price", "deposit", "count",
                "main_amount", "coupon_amount", "again_reduction", "subitems_amount", "deposit_amount", "amount", "paid_amount", "unpaid_amount", "fee", "photos",
                "files", "score", "reply", "remark",
            ]
        )
        main_data, err = EnrollRecordServices.record_add(main_record_params)
        if err:
            transaction.savepoint_rollback(sid)
            return util_response(err=1001, msg=err)

        # 分项表
        subitem_params = format_params_handle(
            param_dict=request_params,
            filter_filed_list=["enroll_record_id", "enroll_subitem_id", "user_id", "subitem_price", "subitem_count", "subitem_amount", ],
            alias_dict={"subitem_price": "price", "subitem_count": "count"}
        )

        enroll_record_id = main_data.get("id", None)
        if not enroll_record_id:
            transaction.savepoint_rollback(sid)
            return util_response(err=1002, msg=err)
        subitem_params["enroll_record_id"] = enroll_record_id

        subitem_data, err = EnrollSubitemRecordService.add(subitem_params)
        if err:
            transaction.savepoint_rollback(sid)
            return util_response(err=1003, msg=err)

        transaction.clean_savepoints()
        # 添加定时器
        clocker = ClockService()
        clocker.add_clock(enroll_id=request_params.get("enroll_id"), user_id=request_params.get("user_id"))
        return util_response()

    @api_view(['DELETE'])
    @request_params_wrapper
    @user_authentication_wrapper
    def un_enroll(self, *args, request_params=None, user_info=None, **kwargs):
        # 用户id获取
        user_id = request_params.get("user_id") or user_info.get("user_id")
        enroll_id = request_params.get("enroll_id")
        if not user_id or not enroll_id:
            return util_response(err=1000, msg="参数错误")

        data, err = EnrollRecordServices.record_del(None, {"user_id": user_id, "enroll_id": enroll_id})

        return util_response()

    @api_view(['GET'])
    @request_params_wrapper
    def enroll_pay_callback(self, *args, request_params=None, **kwargs):
        # 用户id获取
        order_no = request_params.get("order_no", None)
        if not order_no:
            return util_response(err=1000, msg="order_no 不能为空")

        data, err = EnrollServices.bxtx_pay_call_back(order_no)
        if err:
            return util_response(err=1001, msg=err)
        return util_response(msg="回调成功")



from django.views.decorators.http import require_http_methods
from rest_framework.decorators import api_view
from rest_framework.views import APIView

from xj_user.services.user_detail_info_service import DetailInfoService
from xj_user.utils.user_wrapper import user_authentication_force_wrapper
from ..models import Enroll
from ..service.enroll_record_serivce import EnrollRecordServices
from ..service.valuation_service import ValuationService
from ..utils.custom_response import util_response
from ..utils.custom_tool import parse_data, flow_service_wrapper, write_to_log
from ..utils.custom_tool import request_params_wrapper
from ..utils.join_list import JoinList


class RecordAPI(APIView):
    # 添加记录,用户报名
    @api_view(['POST'])
    @user_authentication_force_wrapper
    @request_params_wrapper
    @flow_service_wrapper
    def add(self, *args, user_info=None, request_params=None, **kwargs, ):
        # 提取参数
        if request_params is None:
            request_params = {}
        request_params['user_id'] = user_info.get("user_id")

        # 数据有效性校验
        enroll_id = request_params.get("enroll_id")
        if not enroll_id:
            return util_response(err=1000, msg="enroll_id不能为空")
        enroll_info = Enroll.objects.filter(id=enroll_id).first()
        if not enroll_info:
            return util_response(err=1001, msg="报名ID不正确")

        # 入库之前进行计价
        try:
            request_params.setdefault("again_price", 0)
            valuation_res, err = ValuationService.valuate(
                enroll_rule_group_id=enroll_info.enroll_rule_group_id or 1,
                variables_dict=request_params
            )
            # TODO 这里如果业务变动还是需要改代码，优化到动态处理并放在参数里面
            request_params["initiator_again_price"] = valuation_res.get("initiator_again_price", 0)
            write_to_log(prefix="报名记录添加，计价结果", content="enroll_info:" + str(enroll_info) + "valuation_res:" + str(valuation_res))
        except Exception as e:
            write_to_log(prefix="报名错误", err_obj=e)

        # 添加数据
        data, err = EnrollRecordServices.record_add(request_params)
        if err:
            return util_response(err=1002, msg=err)

        # 报名触发倒计时,取消自动成单
        # clocker = ClockService()
        # data, err = clocker.add_clock(enroll_id, user_id)
        return util_response(data={"id": data.get("id")})

    @require_http_methods(['GET'])
    def list(self, *args, **kwargs, ):
        params = parse_data(self)
        need_pagination = params.get("need_pagination", 1)
        need_pagination = int(need_pagination)
        data, err = EnrollRecordServices.record_list(params=params, need_pagination=need_pagination)
        if err:
            return util_response(err=1000, msg=err)
        user_ids = []
        if need_pagination:
            user_infos = DetailInfoService.get_list_detail({}, user_ids)
            data["list"] = JoinList(data["list"], user_infos, "user_id", "user_id").join()
        else:
            user_infos = DetailInfoService.get_list_detail({}, user_ids)
            data = JoinList(data, user_infos, "user_id", "user_id").join()

        return util_response(data=data)

    @require_http_methods(['GET'])
    def list_v2(self, *args, **kwargs, ):
        params = parse_data(self)
        data, err = EnrollRecordServices.complex_record_list(params=params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @api_view(['DELETE'])
    def record_del(self, *args, **kwargs, ):
        params = parse_data(self) or {}
        pk = kwargs.get("pk") or params.pop("id")
        data, err = EnrollRecordServices.record_del(pk)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @api_view(['PUT'])
    @user_authentication_force_wrapper
    @request_params_wrapper
    @flow_service_wrapper
    def record_edit(self, *args, request_params, **kwargs, ):
        pk = kwargs.get("pk") or request_params.pop("id")
        # 检查报名记录是否可以修改为草稿,防止用户指派完成后，镖师取消报名
        # is_can_cancel, err = EnrollRecordServices.check_can_cancel(pk, change_code=request_params.get("enroll_status_code"))
        # if not is_can_cancel:
        #     return util_response()

        data, err = EnrollRecordServices.record_edit(request_params, pk)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @require_http_methods(['GET'])
    def record_detail(self, *args, **kwargs, ):
        params = parse_data(self) or {}
        pk = kwargs.get("pk", None) or params.pop("id", None) or params.pop("record_id", None) or None
        if not pk:
            return util_response(err=1000, msg="参数错误")
        data, err = EnrollRecordServices.record_detail(pk, search_params=params)

        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data)

    @api_view(['GET'])
    @user_authentication_force_wrapper
    @request_params_wrapper
    @flow_service_wrapper
    def appoint(self, *args, user_info=None, **kwargs, ):
        """
        需求描述：报名可多人报名，不在自动成单，可以手动指派报名
        1.由用户或者客服进行指派工作人员完成任务。
        2.没有被选中的用户，报名状态修改成草稿状态。
        3.主报名项目，则进入代补差价状态。
        :return: response
        """
        params = parse_data(self)
        enroll_id = params.get("enroll_id", None)
        record_id = params.get("record_id", None)
        subitem_id = params.get("subitem_id", None)
        if enroll_id is None or record_id is None or subitem_id is None:
            return util_response(err=1000, msg="参数错误")
        data, err = EnrollRecordServices.appoint(enroll_id, record_id, subitem_id)
        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data, msg="指派成功")

    @api_view(['GET'])
    @user_authentication_force_wrapper
    @request_params_wrapper
    @flow_service_wrapper
    def old_appoint(self, *args, user_info=None, **kwargs, ):
        """
        需求描述：报名可多人报名，不在自动成单，可以手动指派报名
        1.由用户或者客服进行指派工作人员完成任务。
        2.没有被选中的用户，报名状态修改成草稿状态。
        3.主报名项目，则进入代补差价状态。
        :return: response
        """
        params = parse_data(self)
        enroll_id = params.get("enroll_id", None)
        record_id = params.get("record_id", None)
        subitem_id = params.get("subitem_id", None)
        if enroll_id is None or record_id is None or subitem_id is None:
            return util_response(err=1000, msg="参数错误")
        data, err = EnrollRecordServices.old_appoint(enroll_id, record_id, subitem_id)
        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data, msg="指派成功")



# encoding: utf-8
"""
@project: djangoModel->enroll_status_code_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 报名状态码服务
@created_time: 2022/11/13 16:41
"""
from django.db import transaction
from django.db.models import F

from ..models import Enroll, EnrollRecord, EnrollSubitem, EnrollSubitemRecord


class EnrollStatusCodeService():
    @staticmethod
    def batch_edit_code(enroll_id=None, code=None, edit_types=None, params: dict = None):
        """
        流程控制，批量修改状态码服务方法
        :param enroll_id: 报名ID
        :param code: 修改的状态码
        :param edit_types: 需要修改策略列表
        :param params: 需要差异化修改的参数
        :return: data, err
        """
        if edit_types is None:
            edit_types = ["enroll", "enroll_record", "enroll_subitem", "enroll_subitem_record"]
        edit_types = edit_types.split(";") if isinstance(edit_types, str) else edit_types

        # 开启事务进行修改
        sid = transaction.savepoint()
        try:
            if "enroll" in edit_types:
                enroll_query_obj = Enroll.objects.filter(id=enroll_id)
                enroll_query_obj.update(enroll_status_code=code)

            if "enroll_record" in edit_types:
                enroll_record_code = params.get("enroll_record_code") or code
                if params.get("enroll_record_id"):
                    enroll_record_query_obj = EnrollRecord.objects.filter(id=params["enroll_record_id"]).exclude(enroll_status_code=124)
                    enroll_record_query_obj.update(enroll_status_code=enroll_record_code)
                else:
                    enroll_record_query_obj = EnrollRecord.objects.filter(enroll_id=enroll_id).exclude(enroll_status_code=124)
                    enroll_record_query_obj.update(enroll_status_code=enroll_record_code)

            if "enroll_subitem" in edit_types:
                enroll_subitem_code = params.get("enroll_subitem_code") or code
                if params.get("enroll_subitem_id"):
                    enroll_record_query_obj = EnrollRecord.objects.filter(id=params["enroll_subitem_id"]).exclude(enroll_status_code=124)
                    enroll_record_query_obj.update(enroll_status_code=enroll_subitem_code)
                else:
                    enroll_subitem_query_obj = EnrollSubitem.objects.filter(enroll_id=enroll_id).exclude(enroll_subitem_status_code=124)
                    enroll_subitem_query_obj.update(enroll_subitem_status_code=enroll_subitem_code)

            if "enroll_subitem_record" in edit_types:
                enroll_subitem_record_code = params.get("enroll_subitem_record_code") or code
                if params.get("enroll_subitem_record_id"):
                    enroll_record_query_obj = EnrollRecord.objects.filter(id=params["enroll_subitem_record_id"]).exclude(enroll_status_code=124)
                    enroll_record_query_obj.update(enroll_status_code=enroll_subitem_record_code)
                else:
                    enroll_subitem_record_query_obj = EnrollSubitemRecord.objects.annotate(enroll_id=F("enroll_record__enroll_id")) \
                        .filter(enroll_id=enroll_id).exclude(enroll_subitem_status_code=124)
                    enroll_subitem_record_query_obj.update(enroll_subitem_status_code=enroll_subitem_record_code)
            transaction.clean_savepoints()

            return None, None
        except Exception as e:
            transaction.savepoint_rollback(sid)
            return None, str(e)



# encoding: utf-8
"""
@project: djangoModel->custom_merge
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 用户自定义 字典合并
@created_time: 2022/8/16 15:40
"""


class JoinList:
    """
    推荐使用方法：user_info_list = JoinList(user_info_list, user_info_list, "user_id", "user_id").join()
    """

    def __init__(self, l_list, r_list, l_key="id", r_key="id"):
        self.l_key = l_key
        self.r_key = r_key
        self.l_list = l_list or []
        self.r_list = r_list or []

    def join(self):
        r_map = {str(item.pop(self.r_key)): item for item in self.r_list}
        for item in self.l_list:
            if str(item[self.l_key]) in r_map.keys():
                item.update(r_map[str(item[self.l_key])])
        return self.l_list



# encoding: utf-8
"""
@project: djangoModel->enroll_status_code
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 状态码
@created_time: 2022/11/13 16:31
"""
from django.views.decorators.http import require_http_methods
from rest_framework.views import APIView

from xj_user.utils.user_wrapper import user_authentication_force_wrapper
from ..service.enroll_status_code_service import EnrollStatusCodeService
from ..utils.custom_response import util_response
from ..utils.custom_tool import request_params_wrapper, flow_service_wrapper


class EnrollStatusCodeAPI(APIView):
    @require_http_methods(['PUT'])
    @user_authentication_force_wrapper
    @request_params_wrapper
    @flow_service_wrapper
    def batch_edit_code(self, *args, request_params=None, user_info=None, **kwargs, ):
        code = request_params.get("code", None)
        enroll_id = request_params.get("enroll_id", None) or kwargs.get("enroll_id", None)
        edit_types = request_params.get("edit_types", "enroll,enroll_record,enroll_subitem,enroll_subitem_record")
        if not code or not enroll_id or not edit_types:
            return util_response(err=1000, msg="参数错误，code|enroll_id 不能为空")
        data, err = EnrollStatusCodeService.batch_edit_code(
            enroll_id=enroll_id,
            code=code,
            edit_types=edit_types.split(","),
            params=request_params
        )
        if err:
            return util_response(err=1001, msg=err)
        return util_response(data=data)

    @require_http_methods(['GET'])
    @user_authentication_force_wrapper
    @request_params_wrapper
    @flow_service_wrapper
    def ask_pay_status(self, *args, request_params=None, user_info=None, **kwargs, ):
        # 参数处理
        # print("request_params:", request_params)
        success_code = request_params.get("success_code")
        fail_code = request_params.get("fail_code")
        enroll_id = request_params.get("enroll_id")
        edit_types = request_params.get("edit_types", "enroll;enroll_record;enroll_subitem;enroll_subitem_record")
        if not enroll_id:
            return util_response(err=1000, msg="msg:参数错误，报名ID不能为空:tip:非法请求")

        if not success_code or not fail_code:
            return util_response(err=1001, msg="msg:请检亲求参数，或者检查流程配置:tip:非法请求")

        # 询问资金模块是完成了订单 得到 WATING(等待中),SUCCESS（支付成功）,FAIL（支付失败）三种状态
        pay_status = "WATING"
        if pay_status == "WATING":
            return util_response(msg="msg:正在支付中;tip:正在支付中", data=pay_status)
        elif pay_status == "SUCCESS":
            code = success_code
        elif pay_status == "FAIL":
            code = fail_code
        else:
            return util_response(msg="msg:支付模块异常，并没有给到支付状态;tip:正在支付中", data=pay_status)

        # 修改报名的全部状态
        data, err = EnrollStatusCodeService.batch_edit_code(
            enroll_id=enroll_id,
            code=code,
            edit_types=edit_types.split("edit_types"),
            params=request_params
        )
        if err:
            return util_response(err=1002, msg=err)
        return util_response(data=data)



# encoding: utf-8
"""
@project: djangoModel->service_register
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 对外开放服务调用注册白名单
@created_time: 2023/1/12 14:29
"""

import xj_enroll
from xj_enroll.service import subitem_service, enroll_subitem_record_service, enroll_record_serivce, enroll_status_code_service
from .service import enroll_services

# 对外服务白名单
register_list = [
    {
        # 报名添加
        "service_name": "enroll_add",
        "pointer": enroll_services.EnrollServices.enroll_add
    },
    {
        # 报名修改
        "service_name": "enroll_edit",
        "pointer": enroll_services.EnrollServices.enroll_edit
    },
    {
        # 镖师报名
        "service_name": "record_add",
        "pointer": enroll_record_serivce.EnrollRecordServices.record_add
    },
    {
        # 用户指派镖师
        "service_name": "appoint",
        "pointer": enroll_record_serivce.EnrollRecordServices.appoint
    },
    {
        # 报名记录修改
        "service_name": "record_edit",
        "pointer": enroll_record_serivce.EnrollRecordServices.record_edit
    },
    {
        # 报名记录修改
        "service_name": "subitem_record_edit",
        "pointer": enroll_subitem_record_service.EnrollSubitemRecordService.edit
    },
    {
        # 报名记录修改
        "service_name": "subitem_edit",
        "pointer": subitem_service.SubitemService.edit
    },
    {
        # 批量修改状态码
        "service_name": "batch_edit_code",
        "pointer": enroll_status_code_service.EnrollStatusCodeService.batch_edit_code
    },
    {
        # 批余额验收联动
        "service_name": "enroll_check_and_accept",
        "pointer": enroll_services.EnrollServices.enroll_check_and_accept
    },
    {
        # 老版本业务逻辑兼容，批余额验收联动
        "service_name": "old_enroll_check_and_accept",
        "pointer": enroll_services.EnrollServices.old_enroll_check_and_accept
    },

]


# 遍历注册
def register():
    for i in register_list:
        setattr(xj_enroll, i["service_name"], i["pointer"])


if __name__ == '__main__':
    register()
